/** ===========================================================================
	* Config
	* ======
	* Send this into the constructor, instead of long and complex argument lists.
	* We thus extend the JSX API without polluting the JOS API.
	* Use the same for both ObjOut (serialization) and ObjIn (deserialization).
	* Less elegant, and redundant; but simpler to write, maintain and use.
	* 
	* C++ style for setters enables this usage:
	*  Config c = new Config().aliasesUsed(true).IDsUsed(true);
	*
	* Nice: to design to serialize to XML nicely.  For example:
	* Sub-sets of functionality would have their own object.
	*  Config c = (Config) JSX.ObjIn(new FileReader("cyclicConfig.jsx"));
	**/

package JSX;

public class Config {
	public Config() {}	//because default visibility is packageVersion(true);

/** Refactor - used by ObjIn, for mapping class and package names.
	*		Implementations of Refactor avail: ClassRefactor
	*		(You can also make your own implementation).
**/
	public Refactor refactor = null;
	public Config setRefactor(Refactor refactor) {
		this.refactor = refactor;
		return this;
	}


/**	ObjIn - fields missing from classes.
	* Handling NoSuchFieldExceptions:
	* This handles fields in XML which are not found in the class.
	* NOTE: This prevents putFields from working - we could avoid this problem
	* by requiring persistentSerialFields, and returning these with the
	* getAllFields.
	* Q: if I define persistentSerialFields, does that make all other fields
	* transient?
	**/
	//CONSIDER: a better (read: less verbose) name for this?
	public NoSuchFieldHandler noSuchFieldHandler;
	public Config setNoSuchFieldHandler(NoSuchFieldHandler noSuchFieldHandler) {
		this.noSuchFieldHandler = noSuchFieldHandler;
		return this;
	}

//ObjOut - formatting
//Gil Peeter's compact "no formatting" (no unnec whitespace, all one line)
  public boolean formatted = true;	//easy test
	public Config setFormatted(boolean formatted) {
		this.formatted = formatted;
		return this;
	}

//ObjOut - superVersion
	public boolean superVersion = false;	//by default
	public Config superVersion(boolean superVersion) {
		this.superVersion = superVersion;
		return this;
	}

//ObjIn - import, XSLT stylesheet preprocess - filename
	public String importFile;		//Data object - do not change name!
		//Is an external type (memento) better for parameter passing?  We can edit
		//the implementation without changing the external representation.  Or, if
		//change is needed, instead editing the external type, to make a new one
		//altogether (with a different name).  Type safety automatically insures
		//code is written to handle this new external type.
		//Consider that a common informal technique might be to place all the
		//data into a "data object" (stamping?), and send that.  This amounts to
		//an informal external type, and might not have all the advantages of the
		//full method mentioned here.
	public Config importFile(String importFile) {
		this.importFile = importFile;
		return this;
	}

//ObjOut - export, XSLT stylesheet postprocess - filename
//Not yet implemented
/*
	public String exportFile;
	public Config exportFile(String exportFile) {
		this.exportFile = exportFile;
		return this;
	}
*/


//ObjOut - aliases	- this works from the constructor switch.  Show
// *rewire* so that it is controlled from here instead.
/*
	public boolean areAliasesUsed;
	public Config aliasesUsed(boolean areAliasesUsed) {
		this.areAliasesUsed = areAliasesUsed;
		return this;
	}
*/

//ObjOut - ID's  - this really should be implemented...
//Note: if this is switched on, then all Strings will be rendered as
//elements, instead of as attributes, so that their ID can be stated.
// NEED: to standardize this config object!
// (1). Strings are rendered as tags (never as attributes)
// (2). an "alias-ID" attribute is included with every object tag
//  - NOT: null, alias-ref, classes within uper, opt-data nor prim-opt data.
	// public boolean aliasID = true;	// for testing only
	public boolean aliasID = false;	// off by default, because cleaner XML
	public Config aliasID(boolean aliasID) {
		this.aliasID = aliasID;
		return this;
	}


// DTDGenerator
// ------------
//JSX is now configured with a DTDGenerator.  This requires creating the
//DTDGenerator, including it's Writer (same as for ObjOut).
//This might not be the best architecture, but at least it keeps it simple,
//so not too hard to understand, maintain and changet later on...

//If ObjOut is configured with DTDwriter set (ie non-null), then each 
//object-graph written will generate its corresponding DTD to that writer.

//JSX does some inference - rather than account for all 5000+ possible standard
//classes, and all your application classes, JSX instead assumes that the
//objects you write out are representative, and there is at least one
//instance object of each class that you will ever want to write out.
//  NB: This means that you had better ensure that the object graph you use
//  to generate the DTD does indeed includes instances of all the classes 
//  you will ever be interested in.

//Particular usage note:
//If want to write several object graphs, but only need one DTD, you should
//construct a second ObjOut without a DTDwriter.

//Future work:
//It may be better to allow several object graphs to be written, to give the
//best chance of all the classes being included (ie gain not just a
//representative sample, but a comprehensive, definitive one).
//  Option 1: Write the DTD after seeing several graphs, but how to trigger it?
//            It could be the out.close(), but this feels confusing
//  Option 2: We could just append to the old DTD.  Because there is no
//            header information, they can be joined seamlessly.  All that is
//            required is that we don't duplicate information.  This could be
//            tricky, as we use the class list on both sides (as ELEMENTS
//            and field content...).  Solution is to track where we are up to
//            in the list.  **NO** won't work: because the earlier ELEMENTS
//						won't know that their content could include the later classes...
//
// A nicer final option may be to make the DTD be produced *instead* of the
//XML.  It depends on how it is used - and in practice, the overhead of 
//producing the XML is trivial, since making DTD's is typically a one-off
//process.  It just doesn't matter, although it would be "nice".  The real
//goal is to make it *simple* for what needs to be done, and avoiding confusion.

//Future work:
//An alternative is for JSX to enable you to explicitly specify which classes
//you are interested in, instead of JSX gathering this information from the
//object graph.  Perhaps the most convenient would be a combination, as it is
//easy to omit a class which is included (eg indirectly).

//The real danger of explicit specification is omitting the objects that
//a class's fields would typically reference... we could infer these from
//the types of those fields, but it is a risky business - very easy to omit
//some.

// ------NOT IMPLEMENTED AS AT: 2 March 2002--------
	//public DTDGenerator dtdGenerator = null;	//good better naming convention?
	public DTDGenerator dtdGenerator = null;
	public Config setDTDGenerator(DTDGenerator dtdGenerator) {
		this.dtdGenerator = dtdGenerator;
		return this;
	}
/*
	public Config setDTDwriter(java.io.Writer DTDwriter) {
		this.DTDwriter = DTDwriter;
		return this;
	}
*/

/** "autoMemento" means that if a class declares an inner memento, then
	* any serialized classes it encounters will have their names changed to
	* be the memento.  In this way, a Memento can be retrofitted, to enable
	* an old version to be read by a new version, even though the implementation
	* have evolved, and the old version didn't declare a Memento.
	**/
	public boolean autoMemento = true;	// for testing
	public Config setAutoMemento(boolean autoMemento) {
		this.autoMemento = autoMemento;
		return this;	// for chaining
	}
}

